@@ -6,9 +6,10 @@ from tf_metric_learning.utils.constants import *
 class MultiSimilarityLoss(tf.keras.layers.Layer):
     """
     The original implementation was taken from: https://github.com/geonm/tf_ms_loss
+    Use tf.keras.Input(shape=(1), name="input_labels") for input.
     """
 
-    def __init__(self, alpha=2.0, beta=50.0, lamb=1.0, eps=0.1, weight=1.0, **kwargs):
+    def __init__(self, alpha=2.0, beta=50.0, lamb=1.0, eps=0.1, weight=1.0, mining=False, **kwargs):
         super(MultiSimilarityLoss, self).__init__(**kwargs)
 
         self.alpha = alpha
@@ -16,6 +17,7 @@ class MultiSimilarityLoss(tf.keras.layers.Layer):
         self.lamb = lamb
         self.eps = eps
         self.weight = weight
+        self.mining = mining
 
     def get_config(self):
         config = {"alpha": self.alpha, "beta": self.beta, "lamb": self.lamb, "eps": self.eps, "weight": self.weight}
@@ -23,19 +25,11 @@ class MultiSimilarityLoss(tf.keras.layers.Layer):
         return dict(list(base_config.items()) + list(config.items()))
 
     @tf.function
-    def loss_fn(self, embeddings_1, embeddings_2):
-        batch_size = tf.shape(embeddings_2)[0]
-
-        embeddings = tf.concat([embeddings_1, embeddings_2], axis=0)
-        embeddings = tf.nn.l2_normalize(embeddings, axis=1)
-
-        labels = tf.concat([tf.range(batch_size), tf.range(batch_size)], axis=0)
-        labels = tf.reshape(labels, [-1, 1])
-
+    def loss_fn(self, embeddings, labels):
+        batch_size = tf.size(labels)
         adjacency = tf.equal(labels, tf.transpose(labels))
         adjacency_not = tf.logical_not(adjacency)
-
-        mask_pos = tf.cast(adjacency, dtype=tf.float32) - tf.eye(batch_size * 2, dtype=tf.float32)
+        mask_pos = tf.cast(adjacency, dtype=tf.float32) - tf.eye(batch_size, dtype=tf.float32)
         mask_neg = tf.cast(adjacency_not, dtype=tf.float32)
 
         sim_mat = tf.matmul(embeddings, embeddings, transpose_a=False, transpose_b=True)
@@ -44,6 +38,17 @@ class MultiSimilarityLoss(tf.keras.layers.Layer):
         pos_mat = tf.multiply(sim_mat, mask_pos)
         neg_mat = tf.multiply(sim_mat, mask_neg)
 
+        if self.mining:
+            max_val = tf.reduce_max(neg_mat, axis=1, keepdims=True)
+            tmp_max_val = tf.reduce_max(pos_mat, axis=1, keepdims=True)
+            min_val = tf.reduce_min(tf.multiply(sim_mat - tmp_max_val, mask_pos), axis=1, keepdims=True) + tmp_max_val
+
+            max_val = tf.tile(max_val, [1, batch_size])
+            min_val = tf.tile(min_val, [1, batch_size])
+
+            mask_pos = tf.where(pos_mat < max_val + self.eps, mask_pos, tf.zeros_like(mask_pos))
+            mask_neg = tf.where(neg_mat > min_val - self.eps, mask_neg, tf.zeros_like(mask_neg))
+
         pos_exp = tf.exp(-self.alpha * (pos_mat - self.lamb))
         pos_exp = tf.where(mask_pos > 0.0, pos_exp, tf.zeros_like(pos_exp))
 
@@ -57,8 +62,8 @@ class MultiSimilarityLoss(tf.keras.layers.Layer):
         return loss
 
     def call(self, inputs):
-        embeddings_a, embeddings_p = inputs[ANCHOR], inputs[POSITIVE]
-        loss = self.loss_fn(embeddings_a, embeddings_p)
+        embeddings, labels = inputs[ANCHOR], inputs[LABELS]
+        loss = self.loss_fn(embeddings, labels)
         loss = loss * self.weight
 
         self.add_loss(loss)
